@@ -27,8 +27,8 @@
 #                is the re-adjusted value.                                     #
 #                                                                              #
 #                                                                              #
-#   Date: December, 19, 2011                                                   #
-#   Last modification on: May 15, 2012                                         #
+#   Date: December 19, 2011                                                    #
+#   Last modification on: October 16, 2012                                     #
 ################################################################################
 
 Mloglikelihood <- function(p,
@@ -39,8 +39,10 @@ Mloglikelihood <- function(p,
   # ---- Assign the number of frailty parameters 'obs$nFpar' ------------------#
   # ---- and compute Omega for the Positive Stable frailty --------------------#
   
-  if (frailty %in% c("gamma","ingau")) {
+  if (frailty %in% c("gamma", "ingau")) {
     theta <- exp(p[1])
+  } else if (frailty == "lognormal") {
+    sigma <- exp(p[1])
   } else if (frailty == "possta") {
     nu <- exp(-exp(p[1]))
     D <- max(obs$dqi)
@@ -51,7 +53,7 @@ Mloglikelihood <- function(p,
   # ---- Baseline hazard ------------------------------------------------------#
   
   # baseline parameters
-  if (dist == "weibull") {
+  if (dist %in% c("weibull", "inweibull")) {
     pars <- cbind(rho    = exp(p[obs$nFpar + 1:obs$nstr]),
                   lambda = exp(p[obs$nFpar + obs$nstr + 1:obs$nstr]))
     beta <- p[-(1:(obs$nFpar + 2 * obs$nstr))]
@@ -175,6 +177,11 @@ Mloglikelihood <- function(p,
                                             nu=nu, Omega=Omega, 
                                             what="logLT",
                                             correct=correct))
+  } else if (frailty=="lognormal") {
+    logSurv <- mapply(fr.lognormal, 
+                      k=obs$di, s=as.numeric(cumhaz[[1]]), 
+                      sigma=rep(sigma, obs$ncl), 
+                      what="logLT")
   } else if (frailty=="none") {
     logSurv <- mapply(fr.none, s=cumhaz, what="logLT")
   }
@@ -199,6 +206,11 @@ Mloglikelihood <- function(p,
                                                nu=nu, Omega=Omega, 
                                                what="logLT",
                                                correct=correct))
+    } else if (frailty=="lognormal") {
+      logSurvT <- mapply(fr.lognormal, 
+                         k=0, s=as.numeric(cumhazT[[1]]), 
+                         sigma=rep(sigma, obs$ncl), 
+                         what="logLT") 
     } else if (frailty=="none") {
       logSurvT <- mapply(fr.none, s=cumhazT, what="logLT")
     }
